//css_ref System.Transactions;
//css_ref System.Xml;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;
//css_ref Produmex.Sbo.Logex.Paris8.dll;

using System;
using System.Diagnostics;
using System.Reflection;
using System.Transactions;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Resources;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data;

namespace AddUpdateScript
{
	class Program
    {
        static string CONNECTION_STRING = "Server=SZABOZS-PC;CompanyDB=TEST_PMX_56;ExtraDb=PMX_TEST_PMX_56;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=manager;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2012;DBCultureInfo=en-US;LicenseServer=SZABOZS-PC:30000";

        /// <summary>
        /// Used for logging
        /// </summary>
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        static void Main(string[] args)
        {
            try
            {
                LogProvider.SetConsoleAppender("WARN");
                Execute();
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
            }
            if (Debugger.IsAttached)
            {
                Console.WriteLine("Press <Enter>");
                Console.ReadLine();
            }
        }

		private static readonly Assembly s_paris8Assembly = typeof(Produmex.Sbo.Logex.Paris8.SboCommands).Assembly;
		private static string LOCATION_SCRIPTS = "Produmex.Sbo.Logex.Paris8.Scripts.";

		static void Execute()
        {
            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (SboConnection conn = new SboConnection(CONNECTION_STRING))
                {
                    conn.Open(); //Open connection to SAP
                    Console.WriteLine("Connection is open to database");

					const string FORMTYPE = "PAR_OPOL";
					const string QUALITYSTATUS_ITEMID = "EDT_QYS";
					const string WAREHOUSE_ITEMID = "EDT_WH";
					const string WAREHOUSECOLUMN_COLUMNID = "WhsCode";
					const string STORAGELOCATION_ITEMID = "EDT_SL";
					const string STORAGELOCATIONCOLUMN_ITEMID = "MTX_OPOL";
					const string STORAGELOCATIONCOLUMN_COLUMNID = "StorLocCod";
					const string USERQUERY_LOCATIONSFORWAREHOUSE = "PAR_LOCATIONS_FOR_WAREHOUSE";
					const string USERQUERY_LOCATIONSFORWAREHOUSECOLUMN = "PAR_LOCATIONS_FOR_WAREHOUSE_COLUMN";

					PmxDbVersionUpdaterProvider dbvup = new PmxDbVersionUpdaterProvider(conn);

					SboUserQueryCategoryProvider uqcProvider = new SboUserQueryCategoryProvider(conn);

					int categoryCode;
					using (SboUserQueryCategory category = uqcProvider.GetUserQueryCategoryByName(PmxUserQueryCategoryName.PMX_ADDON))
					{
						categoryCode = category.Code;
					}

					int? userQueryID = dbvup.GetUserQuery(PmxUserQueryName.QUALITY_STATUSES);

					if (userQueryID.HasValue)
					{
						dbvup.AddUpdateFormattedSearch(
							FORMTYPE,
							QUALITYSTATUS_ITEMID,
							null,
							userQueryID.Value,
							null,
							false, false);
					}

					int userQueryIDLocations = dbvup.AddUpdateUserQuery(USERQUERY_LOCATIONSFORWAREHOUSE, categoryCode,
						SboCommands.GetQuerySelectLocationForWarehouseGlobal("$[$EDT_WH.0.0]", conn.DbCultureInfo, conn.DbTool));

					dbvup.AddUpdateFormattedSearch(
						FORMTYPE,
						STORAGELOCATION_ITEMID,
						null,
						userQueryIDLocations,
						WAREHOUSE_ITEMID,
						false, false);

					userQueryIDLocations = dbvup.AddUpdateUserQuery(USERQUERY_LOCATIONSFORWAREHOUSECOLUMN, categoryCode,
						SboCommands.GetQuerySelectLocationForWarehouseGlobal("$[$MTX_OPOL.WhsCode.0]", conn.DbCultureInfo, conn.DbTool));

					dbvup.AddUpdateFormattedSearch(
						FORMTYPE,
						STORAGELOCATIONCOLUMN_ITEMID,
						STORAGELOCATIONCOLUMN_COLUMNID,
						userQueryIDLocations,
						WAREHOUSECOLUMN_COLUMNID,
						false, false);
				}
				
                //Complete transaction
                scope.Complete();
            }

            //PMX connection
            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (PmxDbConnectionSql dbConn = new PmxDbConnectionSql(CONNECTION_STRING))
                {
                    dbConn.Open();

                    Console.WriteLine("Tables");
					Console.WriteLine("Views");

					dbConn.DbTool.CreateUpdateView("PAR_OPEN_PURCHASE_ORDER_LINES", ResourceHelper.ReadResourceContentAsString(s_paris8Assembly, LOCATION_SCRIPTS + "Views.PAR_OPEN_PURCHASE_ORDER_LINES.sql"));

					//Complete transaction
					scope.Complete();
                }
            }

			Console.WriteLine("Done ...");
        }
    }
}
